pathsToBinary <- function(ypaths) {
  makeBin <- function(pathGenes,allGenes) return(as.numeric(allGenes %in% pathGenes$genes))

  # if there are response labels
  if (!is.null(names(ypaths$paths))) { 
    all.genes <- c()
    path.data <- NULL

    all.genes <- unique(unlist(lapply(unlist(ypaths$paths,FALSE),"[[","genes")))
    sidx <- which(all.genes == "s")
    tidx <- which(all.genes == "t")
    all.genes <- all.genes[c(sidx,(1:length(all.genes))[-c(sidx,tidx)],tidx)]

    resp <- c()
    for (p in 1:length(ypaths$paths)) {
      binpaths <- data.frame(t(sapply(ypaths$paths[[p]],makeBin,allGenes = all.genes)))
      names(binpaths) <- all.genes

      if (is.null(path.data)) path.data <- binpaths
      else path.data <- rbind(path.data,binpaths)

      resp <- c(resp,rep(names(ypaths$paths)[p],length(ypaths$paths[[p]])))
    }
    pl <- sapply(ypaths$paths,length)
    m.idx <- as.matrix(cbind(rep(1,sum(pl)),
                         rep(1:length(ypaths$paths),pl),
                         as.numeric(sequence(pl))))

    return(list(paths = path.data,y = as.factor(resp), pidx = m.idx))
  } else {
    all.genes <- unique(unlist(lapply(ypaths$paths,"[[","genes")))
    sidx <- which(all.genes == "s")
    tidx <- which(all.genes == "t")
    all.genes <- all.genes[c(sidx,(1:length(all.genes))[-c(sidx,tidx)],tidx)]

    binpaths <- data.frame(t(sapply(ypaths$paths,makeBin,allGenes = all.genes)))

    pl <- sapply(ypaths$paths,length)
    m.idx <- as.matrix(cbind(rep(1,sum(pl)),
                         as.numeric(sequence(pl))))

    names(binpaths) <- all.genes
    return(list(paths = binpaths,pidx = m.idx))
  }
}

pathClassifier <- function(paths,target.class,M,alpha=1,lambda=2,hme3miter = 100,plriter = 1,init = "random") {
    if ((target.class %in% levels(paths$y)) == FALSE) stop(paste("Cannot find",target.class,"in paths$y object"))
    y <- ifelse(paths$y == target.class,1,0) 
    x <- paths$paths

    # remove constant columns to train HME3M
    varying.cols <- which(sd(x) != 0)
    tr.x <- x[varying.cols]

    if (init == "3M") {
        cat("\nRunning initial 3M model")
        # initialize with a 3M model
        pclust <- pathCluster(list(paths = tr.x),M)
        pk <- pclust$proportions
        theta <- as.matrix(pclust$theta)
        beta <- matrix(0,nrow(theta),ncol(theta))
        pmx <- as.matrix(pclust$h) 
        fits <- matrix(0.5,nrow(tr.x),ncol = M)
    
        pkm <- matrix(pk,nrow=nrow(tr.x),ncol = M,byrow = TRUE)
        hij <- pkm*pmx*fits/rowSums(pkm*pmx*fits)
    } else {
        # random initialization
        clusters <- sample(1:M,nrow(x),replace = TRUE)
        pk <- rep(1/M,M)

        theta <- as.matrix(aggregate(tr.x,by = list(clusters),mean)[-1])
        pmx <- matrix(0,nrow(tr.x),M) 
        for (k in 1:nrow(theta)) {
            res <- as.matrix(tr.x) %*% diag(as.double(theta[k,])) 
            res[tr.x == 0] <- NA
            pmx[,k] <- apply(res,1,prod,na.rm = TRUE)
        }

        beta <- matrix(0,nrow(theta),ncol(theta))
        fits <- matrix(0.5,nrow(tr.x),ncol = M)

        pkm <- matrix(pk,nrow=nrow(tr.x),ncol = M,byrow = TRUE)
        hij <- pkm*pmx*fits/rowSums(pkm*pmx*fits)
    }

	fit <- .C("hme3m_R",
		y = as.double(y),
		x = as.double(as.matrix(tr.x)),
		m = as.integer(M),
		lambda = as.double(lambda),
		alpha = as.double(alpha),
		nrow = as.integer(nrow(tr.x)),
		ncol = as.integer(ncol(tr.x)),
		hme3miter = as.integer(hme3miter),
		plriter = as.integer(plriter),
		H = as.double(hij),
		PATHPROBS = as.double(pmx),
		PLRPRE = as.double(fits),
		THETA = as.double(theta),
		BETA = as.double(beta),
		PROPORTIONS = as.double(pk),
		HMEPRE = double(nrow(tr.x)),
		LIKELIHOOD = double(hme3miter))

    theta <- matrix(NA,nrow = M,ncol = ncol(x))
    theta[,c(1,ncol(theta))] <- 1
	t.complete <- matrix(as.double(fit$THETA),nrow = M,ncol = ncol(tr.x) ,byrow = TRUE)
    theta[,varying.cols] <- t.complete # add back the constant columns
    theta <- data.frame(theta)

    beta <- matrix(NA,nrow = M,ncol = ncol(x))
    b.complete <- matrix(as.double(fit$BETA),nrow = M,ncol = ncol(tr.x) ,byrow = TRUE)
    beta[,varying.cols] <- b.complete # add back the constant columns
    beta <- data.frame(beta)
	names(beta) <- names(theta) <- names(x)

	hij <- matrix(fit$H,nrow(x),M) 
	fits <- matrix(fit$PLRPRE,nrow(x)) 
	#perf <- compROC(y,fit$HMEPRE)$auc

    # cluster labels
    zm <- apply(hij,1,max)
    cl <- apply(hij == zm,1,which)
    if (is.list(cl)) {
        cat("\nMultiple cluster labels can be assigned to some paths: some clusters might not be valid")
        clusters <-  paste("M",sapply(cl,"[[",1),sep = "")
    } else clusters <- cl

    pmx <- matrix(fit$PATHPROBS,nrow(x),M) 
    pmx <- pmx/rowSums(pmx)
	output <- list(h = hij,
		theta = theta,
		beta = beta,
		proportions = fit$PROPORTIONS,
		posterior.probs = fit$HMEPRE,
		likelihood = fit$LIKELIHOOD[1:fit$hme3miter],
		plr.probabilities = fits,
        path.probabilities = pmx,
		params = list(alpha = alpha,lambda = lambda,M = M),
		y = y,
		#perf = perf,
        label = ifelse(fit$HMEPRE > 0.5,1,0),
        component = clusters)

	return(output)
}

predictPathClassifier <- function(mix,newdata) {
    pexp <- matrix(0,nrow=nrow(newdata),ncol = nrow(mix$theta))
    pmx <- matrix(0,nrow=nrow(newdata),ncol = nrow(mix$theta))
    pk <- mix$proportions

    tt <- mix$theta
    tt[is.na(mix$theta)] <- 1
    tb <- mix$beta
    tb[is.na(mix$beta)] <- 0
    for (k in 1:nrow(mix$theta)) {
        pred <- as.matrix(newdata) %*% t(tb[k,])
        pexp[,k] <- 1/(1+exp(-pred))
        res <- sweep(as.matrix(newdata),2,t(tt[k,]),FUN = "*")
		res[newdata == 0] <- NA
		pmx[,k] <- apply(res,1,prod,na.rm = TRUE)
    }
    h <- sweep(pmx*pexp,2,t(pk),FUN = "*")
    h <- h/rowSums(h)

    # cluster labels
    zm <- apply(h,1,max)
    cl <- apply(h == zm,1,which)
    if (is.list(cl)) {
        cat("\nMultiple cluster labels for some paths: some clusters might not be valid")
        clusters <-  paste("M",sapply(cl,"[[",1),sep = "")
    } else clusters <- cl

    post <- rowSums(pmx*pexp/rowSums(pmx))

    return(list(h = h,
                posterior.probs = post,
                label = ifelse(post > 0.5,1,0),
                component = clusters,
                path.probabilities = pmx/rowSums(pmx),
                plr.probabilities = pexp))
}

plotPathClassifier <- function(mix) {   
    palette("default") 

    layout(c(1,2),width = c(1,1),height = c(0.7,0.3))
    plotPathROC(mix)

    plot(na.omit(mix$likelihood),type="l",col=2,
		xlab="EM Iteration",ylab="Conditional\nLog-Likelihood",main="Likelihood Convergence",cex = 0.7)
}

plotPathClassifierPath <- function(paths,obj,m,tol = NULL) {
    pp <- obj$theta[m,]
    fidx <- 1:length(pp)
    if (!is.null(tol)) fidx <- which((pp >= tol) | is.na(pp))
    pp <- pp[fidx]

    g <- names(pp)
    tg <- g[-which(g %in% c("s","t"))]
    idx <- sort(sapply(strsplit(tg,":"),"[[",5),index.return = TRUE)$ix
    g <- c("s",tg[idx],"t")

    gc <- strsplit(g,":")[-c(1,length(g))]
    frt <- c("",paste(sapply(gc,"[[",2),sapply(gc,"[[",4),sapply(gc,"[[",3),sep = "-"),"")
    gn <- c("s",sapply(gc,"[[",1),"t")
    pname <- sapply(gc,"[[",5)
    pcol <- c(0,as.numeric(as.factor(pname)),0)
    
    y <- paths$y
    h <- obj$h[,m]
    pp <- pp[c(1,idx+1,length(pp))]
    x <- paths$paths[fidx][c(1,idx+1,length(pp))]
    bp <- obj$beta[m,][fidx][c(1,idx+1,length(pp))]

    palette("default") 
    mpar <- par()$mar

    sy <- sort(h,index.return = TRUE)
 	ypred <- as.numeric(h > 0.5)

    n <- layout(matrix(c(1,2,4,5,5,3),3,2),
		heights = c(.15,0.15,.7,.33,.7),
		widths =  c(.7,.3,.7,.7,.3), TRUE)

    par(mar = c(0,8,2,0),pty = "m")
    xpos <- barplot(height = as.numeric(bp),space = 0,xlim = c(0,ncol(x)),xaxs = "i",
                ylab = "beta", col = pcol %% 5 + 1)
    abline(h = 0,lwd = 2)

	par(mar = c(0,8,1,0),pty = "m")
	xpos <- barplot(height = as.numeric(pp),space = 0,xlim = c(0,ncol(x)),xaxs = "i",ylim = c(0,1),
	ylab = "theta", col = pcol %% 5 + 1)
    abline(h = 0,lwd = 2)

	par(mar = c(30,0,0,4))	
	plot(NA,xlim = c(0,1),ylim = c(1,nrow(x)),
		xlab = paste("Probability P(M=",m,"|x,y)",sep =""),ylab = "",yaxs = "i",type = "l",axes = F)
    points(x = h[sy$ix],y = 1:nrow(x),type = "l",col = 1)
	abline(v = 0.5,col = 2,lwd = 2)
	abline(v = 0,col = 1,lwd = 2)
	axis(side = 1,at = seq(0,1,1/10),las = 1)
	abline(h = nrow(x)+1)

	ystats <- as.numeric(summary(as.factor(ypred)))
	ylab <- c("","","","Not a\nMember","Member")
	ytick <- c(1,nrow(x),ystats[1],ystats[1]/2,ystats[2]/2 + ystats[1])
	
	par(mar = c(30,8,0,0))
	px <- x[sy$ix,]
    px[px == 0] <- NA
    ry <- matrix(rep(sy$x < 0.5,ncol(x)),nrow(x),ncol(x))
	px[ry] <- px[ry] + 1
	image(x = xpos,y = 1:nrow(x),as.matrix(t(px)),axes = F,xlab = "",ylab = "",xaxs = "i",yaxs = "i",col = cm.colors(2))
	axis(side = 2,at = ytick,labels = ylab,las = 2)
    mtext(gn,side = 1,at = xpos,cex = 0.6,las =2,line = 0.5,
        col = pcol %% 5 + 1 )
    mtext(frt,side = 1,at = xpos,cex = 0.6,las = 2,line = 5,
        col = pcol %% 5 + 1)
    pd <- cbind(xpos[-c(1,length(xpos))],pcol[-c(1,length(pcol))])
    pht <- aggregate(pd,list(pname),median)
    mtext(pht[,1],side = 1,at = pht[,2],cex = 0.6,las = 2,line = 15,col = as.numeric(as.factor(pht[,3])) %% 5 + 1)
    mtext("All Paths\nSorted By Component Membership",line = 5,side = 2)

	par(mar = c(1,1,6,6))
    rocs <- compROC(obj$y,obj$h[,m]) 
    plot(NA,xlim = c(0,1),ylim = c(0,1),axes = FALSE)
    abline(0,1)
    lines(x = rocs$fnr,y = rocs$tpr,type = "l",col = 2,lwd = 2)
    axis(side = 3,at = seq(0,1,1/5),labels = seq(0,1,1/5),cex= 0.5)
    mtext("FNR",3,line = 2,cex = 0.7)
    axis(side = 4,at = seq(0,1,1/5),labels = seq(0,1,1/5),cex = 0.5)
    mtext("TPR",4,line= 2,cex = 0.7)
    mtext(paste("ROC for Path",m,"\nAUC = ",round(rocs$auc,3)),line = 3,cex = 0.7)

    mtext(paste("Structure of Path",m),side = 3,outer = TRUE,line = -2,cex = 2)

    layout(1)
    par(mar = mpar)
}


 
