pathCluster <- function(paths,M,iter = 1000) {
  x <- paths$paths

  # remove constant columns
  varying.cols <- which(sd(x) != 0)
  tr.x <- x[varying.cols]

  zcluster <- sample(1:M,nrow(tr.x),replace = T)
  ptheta <- as.matrix(aggregate(tr.x,list(zcluster),mean)[-1])

  pk <- rep(1/M,M)

  pmx <- matrix(0,nrow(tr.x),M) 
  for (k in 1:M) {
    res <- as.matrix(tr.x) %*% diag(as.double(ptheta[k,])) 
	  res[tr.x == 0] <- NA
	  pmx[,k] <- apply(res,1,prod,na.rm = TRUE)
  }

  pkm <- matrix(pk,nrow=nrow(tr.x),ncol = M,byrow = TRUE)
  hij <- pkm*pmx/rowSums(pkm*pmx)

  fit <- .C("pathMix",
    X = as.integer(as.matrix(tr.x)),
    M = as.integer(M),
    NOBS = as.integer(nrow(tr.x)),
    NX = as.integer(ncol(tr.x)),
    ITER = as.integer(iter),
    H = as.double(hij),
    THETA = as.double(t(ptheta)),
    PROPORTIONS = as.double(pk),
    LIKELIHOOD = double(iter))

  posterior.probs = data.frame(matrix(fit$H,ncol = M))
  names(posterior.probs) <- paste("M",1:M,sep = "")

  theta <- matrix(NA,nrow = M,ncol = ncol(x))
  t.complete <- matrix(as.double(fit$THETA),nrow = M,ncol = ncol(tr.x),byrow = TRUE)
  theta[,varying.cols] <- t.complete
  theta[,c(1,ncol(theta))] <- 1
  theta <- data.frame(theta)
  names(theta) <- names(x)

  ll <- fit$LIKELIHOOD[1:fit$ITER]

  # cluster labels
  zm <- apply(posterior.probs,1,max)
  cl <- apply(posterior.probs == zm,1,which)
  if (is.list(cl)) {
        cat("\nMultiple cluster labels for some paths, some clusters might not be valid\n")
        clusters <-  paste("M",sapply(cl,"[[",1),sep = "")
  } else clusters <- cl
     
  return(list(h = posterior.probs,
              labels = clusters,
              theta = theta,
              proportions = fit$PROPORTIONS,
              likelihood = ll,
              params = list(M = M)))
}

predictPathCluster <- function(pfit,newdata) { 
  pmx <- matrix(0,nrow=nrow(newdata),ncol = nrow(pfit$theta))
  tt <- pfit$theta
  tt[is.na(tt)] <- 1

  for (k in 1:nrow(pfit$theta)) {
	  res <- as.matrix(newdata) %*% diag(as.numeric(tt[k,])) 
	  res[newdata == 0] <- 1
	  pmx[,k] <- pfit$proportions[k] * apply(res,1,prod)
  }
  pmx <- data.frame(pmx/rowSums(pmx))
  names(pmx) <- paste("M",1:ncol(pmx),sep = "")

  # cluster labels
  zm <- apply(pmx,1,max)
  cl <- apply(pmx == zm,1,which)
  if (is.list(cl)) {
        cat("\nMultiple cluster labels for some paths: some clusters might not be valid\n")
        clusters <-  sapply(cl,"[[",1) # just take the first because they are all just as likely!
  } else clusters <- cl

  return(list(labels = clusters,h = pmx))
}

plotPathClusterPath <- function(paths,obj,m,tol = NULL) {
    pp <- obj$theta[m,]
    fidx <- 1:length(pp)
    if (!is.null(tol)) fidx <- which(pp >= tol | is.na(pp))
    pp <- pp[fidx]

    g <- names(pp)
    tg <- g[-which(g %in% c("s","t"))]
    idx <- sort(sapply(strsplit(tg,":"),"[[",5),index.return = TRUE)$ix
    g <- c("s",tg[idx],"t")

    gc <- strsplit(g,":")[-c(1,length(g))]
    frt <- c("",paste(sapply(gc,"[[",2),sapply(gc,"[[",4),sapply(gc,"[[",3),sep = "-"),"")
    gn <- c("s",sapply(gc,"[[",1),"t")
    pname <- sapply(gc,"[[",5)
    pcol <- c(0,as.numeric(as.factor(pname)),0)

    mpar <- par()$mar

    pp <- pp[c(1,idx+1,length(pp))]
    x <- paths$paths[fidx][c(1,idx+1,length(pp))]

	h <- obj$h[,m]
    sh <- sort(h,index.return = TRUE)

	nl <- layout(matrix(c(1,4,3,2),2,2,byrow=T),
		heights = c(.2,.8,.8,.2),
		widths =  c(.8,.2,.8,.2),TRUE)

	par(mar = c(0,8,4,0),pty = "m")
	xpos <- barplot(height = as.numeric(pp),space = 0,xlim = c(0,ncol(x)),xaxs = "i",ylim = c(0,1),
	ylab = "Gene Probabilities", col = pcol %% 5 + 1)	
    abline(h = 0)

	par(mar = c(25,0,0,4))	
	plot(x = sh$x,y = 1:nrow(x),xlim = c(0,1),ylim = c(1,nrow(x)),
		   xlab = paste("P(M = ",m,"|X)",sep =""),ylab = "",yaxs = "i",type = "l",axes = F)
	abline(v = 0.5,col = 2,lwd = 2)
	abline(v = 0,col = 1,lwd = 2)
	axis(side = 1,at = seq(0,1,1/10),las = 1)
	abline(h = nrow(x)+1)

    hp <- as.factor(as.numeric(h > 0.5))
    ystats <- as.numeric(summary(hp))
    ylab <- c("","","","Not a\nMember","Member")
    ytick <- c(1,nrow(x),ystats[1],ystats[1]/2,ystats[2]/2 + ystats[1])

	par(mar = c(25,8,0,0))
    x[x == 0] <- NA
    x[h < 0.5,] <- x[h < 0.5,] + 1
	px <- x[sh$ix,]
	image(x = xpos,y = 1:nrow(x),as.matrix(t(px)),axes = F,ylab = "",xlab = "",xaxs = "i",yaxs = "i",col = cm.colors(2))
	axis(side = 2,at = ytick,labels = ylab,las = 2)
    mtext("All Paths\nSorted By Component Membership",line = 5,side = 2)

    mtext(gn,side = 1,at = xpos,cex = 0.6,las =2,line = 0.5,
        col = pcol %% 5 + 1)
    mtext(frt,side = 1,at = xpos,cex = 0.6,las = 2,line = 4.5,
        col = pcol %% 5 + 1)
    pd <- cbind(xpos[-c(1,length(xpos))],pcol[-c(1,length(pcol))] %% 5 + 1)
    pht <- aggregate(pd,list(pname),median)
    mtext(pht[,1],side = 1,at = pht[,2],cex = 0.6,las = 2,line = 12,
            col = pht[,3])
    mtext(paste("Structure of Path",m),side = 3,outer = TRUE,line = -2,cex = 2)

    layout(1)
    par(mar = mpar)
}

