plotComponents <- function(fit,tol=0,cluster = TRUE) {
    if (cluster) ht <- hclust(dist(fit$theta),method = "complete")

    z1 <- fit$theta
    z1[z1 <= tol] <- NA
    z1[z1 == 0] <- NA
    idx <- c(1,which(colSums(is.na(z1)) == fit$params$M),ncol(z1))
    z1 <- z1[,-idx]

    g <- names(z1)
    sidx <- sort(sapply(strsplit(g,":"),"[[",5),index.return = TRUE)$ix
    g <- g[sidx]
    z1 <- z1[,sidx]
    cnames <- paste("M",1:fit$params$M,sep = "")
    if (cluster) cnames <- cnames[ht$order]

    gc <- strsplit(g,":")
    frt <- c(paste(sapply(gc,"[[",2),sapply(gc,"[[",4),sapply(gc,"[[",3),sep = "-"))
    gn <- c(sapply(gc,"[[",1))
    pname <- sapply(gc,"[[",5)
    pcol <- as.numeric(as.factor(pname)) %% 5 + 1

    par(mar = c(2,24,4,4))
    if (cluster) z1 <- z1[ht$order,]
    zh <- image(x = 1:nrow(z1),y = 1:ncol(z1),z = as.matrix(z1),
        axes = FALSE,xaxs = "i",yaxs = "i",xlab = "",ylab = "",col = heat.colors(5))
    abline(h = 0:ncol(z1) + 0.5,lwd = 0.5)
    abline(v = 0:nrow(z1) + 0.5,lwd = 2)
    pd <- cbind(1:length(pcol),pcol)
    pht <- aggregate(pd,list(pname),median)
    mtext(gn,side = 4,at = 1:length(g),cex = 0.7,las =2,line = 0.5,col = pcol)
    mtext(frt,side = 2,at = 1:length(frt),cex = 0.7,las = 2,line = 1,col = pcol)
    mtext(pht[,1],side = 2,at = pht[,2],cex = 0.8,las = 2,line = 8, col = as.numeric(pht[,3]),font = 2)
    mtext(cnames,side = 3,at = 1:fit$params$M,font = 2,cex = 0.8,las = 2,line = 0.5)
    for (i in 1:nrow(z1)) {idx <- which(!is.na(z1[i,])) ; text(x = i,y = idx,round(z1[i,][idx],2),cex = 0.75) }
}

extractPathNetwork <- function(ypaths,feature = "pathway",pathweights = NULL,tol = 0.05) {
    
    getPathEdgeList <- function(i,ypath,nodes,yl,fidx,pathweights = NULL) {
        path <- ypath[[i]]        
        pg <- path$genes
        pg <- pg[-c(1,length(pg))]
        if (fidx != 2) pg <- c("s",sapply(strsplit(pg,":"),"[[",fidx),"t")
        else pg <- c("s",sapply(strsplit(pg,":"),"[[",fidx),strsplit(pg[length(pg)],":")[[1]][[3]],"t")
        w <- rep(pathweights[i],length(pg))
        pg <- match(pg,nodes)
        ed <- data.frame(from = pg[1:(length(pg)-1)],
                         to = pg[2:length(pg)],
                         weight = w[2:length(pg)])
        return(ed)
    }
    genepaths <- lapply(unlist(ypaths$paths,recursive = FALSE),"[[","genes")

    fidx <- which(feature == c("gene","compound","","reaction","pathway"))
    if (is.null(pathweights)) pathweights <- rep(1,length(genepaths))
    pathweights[pathweights < tol] <- 0 # set all paths below the tolerance to 0 

    # create gene network from paths
    genepaths <- lapply(unlist(ypaths$paths,recursive = FALSE),"[[","genes")
    nodes <- unique(unlist(genepaths))
    nodes <- nodes[-which(nodes %in% c("s","t"))]
    if (fidx != 2) nodes <- c("s",unique(sapply(strsplit(nodes,":"),"[[",fidx)),"t")
    else {nodes <- c("s",unique(sapply(strsplit(nodes,":"),"[[",2)),unique(sapply(strsplit(nodes,":"),"[[",3)),"t")}
    edges <- NULL
    lto <- cumsum(sapply(ypaths$path,length))
    lfrom <- 1
    for (yl in 1:length(ypaths$paths)) {     
        ed <- do.call("rbind",lapply(1:length(ypaths$paths[[yl]]),getPathEdgeList,
                                ypath = ypaths$paths[[yl]],
                                nodes = nodes,
                                yl = names(ypaths$paths)[yl],
                                fidx = fidx,
                                pathweights = pathweights[lfrom:lto[yl]]))
        lfrom <- lto[yl] + 1
        if (is.null(edges)) edges <- ed
        else edges <- rbind(edges,ed)
    }

    # merge edges that occur over multiple classes
    edges <- data.frame(edges)
    un.edges <- unique(edges[1:2])
    merged.edges <- NULL
    dup.idx <- c()
    for (i in 1:nrow(un.edges)) {
        ufrom <- un.edges$from[i]; uto = un.edges$to[i]
        idx <- which(edges$from == ufrom & edges$to == uto)
        dup.idx <- c(dup.idx,idx)

        # take the most frequent label over all paths with a weight > 0
        lidx <- which(edges[idx,]$weight > 0)

        ed <- data.frame(from = ufrom,to = uto,weight = sum(edges$weight[idx]))
        if (is.null(merged.edges)) merged.edges <- ed
        else merged.edges <- rbind(merged.edges,ed)
    }
    edges <- edges[-dup.idx,]
    edges <- rbind(edges,merged.edges)
    dimnames(edges)[[1]] <- 1:nrow(edges)

    node.annotation <- nodes

    return(list(edges = edges[1:2],weights = edges$weight,node.ann = node.annotation)) 
}

plotPathNetwork <- function(pnet,tol = 0,mtitle = "") {
    haslib <- require(Rgraphviz)
    if (haslib == FALSE) stop("To plot the network please install the Rgraphviz package.")

    #remove self edges
    edges <- data.frame(pnet$edges)
    w <- pnet$weights
    pidx <- which(edges$from == edges$to)
    if (length(pidx) > 0) { edges <- edges[-pidx,]; w <- w[-pidx]} 
    
    g1 <- ftM2graphNEL(ft = as.matrix(edges), edgemode="directed")

    # remove 0 edges and extract largest connected component
    ze <- which(w/max(w) < tol)
    if (length(ze) > 0) {
        g1 <- removeEdge(from = as.character(edges$from[ze]),to = as.character(edges$to[ze]),g1)
    }
    cnl <- unique(c(connComp(g1)[[1]],nodes(g1)[length(nodes(g1))]))
    g1 <- subGraph(cnl,g1)   

    # Node labels - cater for long pathway names
    nn <- pnet$node.ann[as.integer(cnl)]
    for (i in 1:length(nn)) {
        lbl <- strsplit(nn[i]," ")[[1]]
        ngrp <- as.numeric(cut(1:length(lbl),2))
        if (length(lbl) > 4)  ngrp <- as.numeric(cut(1:length(lbl),3))
        str <- ""
        for (j in 1:max(ngrp)) {
            s1 <- paste(lbl[ngrp == j],collapse = " ")
            if (j < max(ngrp)) str <- paste(str,s1,"\n",sep = "")
            else str <- paste(str,s1,sep = "")
        }
        nn[i] <- str
    }
    nn[1] <- "start"
    nn[length(nn)] <- "end"
    nAttrs <- list()
    nAttrs$label <- nn
    nAttrs$shape <- c("circle",rep("box",length(nn)-2),"circle")
    nAttrs$fill <- c("lightgreen",rep("lightblue",length(nn)-2),"pink")
    nAttrs$fixedsize <- rep(FALSE,length(nn))
    nAttrs$fontsize <- rep(15,length(nn))
    names(nAttrs$fill) <- names(nAttrs$label) <- names(nAttrs$fontsize) <- 
    names(nAttrs$shape) <- names(nAttrs$fixedsize) <- nodes(g1)

    # edge colours
    enames <- apply(edges,1,paste,collapse = "~")
    ee <- rep("red",length(enames))
    w <- as.numeric(cut(w,10))/2 
    eAttrs <- list()
    eAttrs$col <- ee
    eAttrs$lwd <- w 
    eAttrs$arrowType <- rep("normal",length(ee))
    names(eAttrs$lwd) <- names(eAttrs$col) <- names(eAttrs$arrowType) <- enames

    # define start and end subgraphs
    sg <- list()
    sg[[1]] <- list(graph = subGraph("1",g1),cluster=TRUE,attrs=c(rank="min"))
    sg[[2]] <- list(graph = subGraph(nodes(g1)[length(nodes(g1))],g1),cluster=TRUE,attrs=c(rank="max"))

    # global attributes
    Attrs <- list(graph = list(rankdir = "LR",compound = TRUE,clusterrank = "source",remincross = TRUE,group = TRUE,main = mtitle))
    
    nodeRenderInfo(g1) <- nAttrs
    edgeRenderInfo(g1) <- eAttrs
    g1 <- layoutGraph(g1,layoutType = "dot",subGList = sg,nodeAttrs = nAttrs,attrs = Attrs)
    renderGraph(g1)
}

compROC <- function(y,yprob) {
    tpr <- c(0)
    fnr <- c(0)
    for (i in seq(0,1,1/1000)) {
        tpr <- c(tpr, sum(y == 1 & yprob >= i)  / sum(y == 1) ) 
        fnr <- c(fnr, sum(y == 0 & yprob >= i) /  sum(y == 0) )
    }
    trap.rule <- function(x, y) {
        idx <- 2:length(x)
        dx <- x[idx] - x[idx-1] 
        dy <- y[idx] + y[idx-1] 
        auc <- dx%*%dy / 2
        return( auc )
    }
    # The probabilites can get sparse and actual 0's and 1's occur!  
    # This causes problems with the AUC.
    if (sum(tpr == 0) > 0) fnr[tpr == 0] <- max(fnr[tpr == 0])
    if (sum(fnr == 0) > 0)tpr[fnr == 0] <- max(tpr[fnr == 0])

    fnr <- sort(fnr,index.return = TRUE)
    tpr <- tpr[fnr$ix]
    auc <- trap.rule(x = fnr$x,y = tpr)

    return(list(tpr = tpr,fnr = fnr$x,auc = auc))
}

plotPathROC <- function(pfit) {
    palette("default")

    plot(NA,xlim = c(0,2),ylim = c(0,1),axes = FALSE,
        ylab = "True Positive Rate", xlab = "False Positive Rate",
        main ="ROC Curve For Each HME3M Component")
    
    y <- pfit$y

    auc <- c()
    mpre <- pfit$h
    yprob <- cbind(mpre,pfit$posterior.probs)
    auc <- c()
    for (j in 1:ncol(data.frame(yprob))) {  
        zroc <- compROC(y,yprob[,j])
        lines(x = zroc$fnr, y = zroc$tpr,col = j,lwd = 2)
        auc <- c(auc,zroc$auc)
    }
    axis(1,seq(0,1,0.1),seq(0,1,0.1),pos = 0)
    axis(2,seq(0,1,0.1),seq(0,1,0.1),pos = 0)
    lines(x = c(0,1),y = c(1,1))
    lines(x = c(1,1),y = c(0,1))
    lines(x = c(0,1),y = c(0,1),lwd = 1,lty = "dashed")
    mtext("False Positive Rate",side = 1,line = 2,adj = .2)
    lnames <- c(paste("M = ",1:ncol(pfit$h)),"Complete")
    lnames <- paste(lnames,"(AUC =",round(auc,3),")",sep = "")
    legend(x = 1.05,y = .95,lnames,col = 1:ncol(yprob),lty = rep(1,ncol(yprob)),lwd = 2,bg = "white",cex = 0.8)
}

