\name{computeGeneNetwork}
\alias{computeGeneNetwork}
\title{Converts a KEGG compound network object to a gene network}
\description{Specify the start and end compounds within a \code{getKEGG} object and convert to an annotated gene-gene network.}

\usage{
geneNetwork <- computeGeneNetwork(kegg.obj, start = NULL, end = NULL) 
}
\arguments{
\item{kegg.obj}{A compound network computed from \code{\link{getKEGG}}.}
\item{start}{A character vector of start compounds. If \code{NULL} the code will use for all valid compounds.  If no valid compounds are found the function will not run.}
\item{end}{A character vector of end compounds.  If \code{NULL} the code will use for all valid compounds.  If no valid compounds are found the function will not run.}
}
\value{
A list with the following items:
\tabular{ll}{
\code{nodes} \tab A list of all genes found within the network. See \code{details} section for a complete description.\cr
\code{edges} \tab A \code{data.frame} of the edge list. See \code{details} section for a complete description.\cr
\code{start} \tab The list of start compounds used. \cr
\code{end} \tab The list of end compounds used.
}
}
\details{

The returned \code{geneNetwork$nodes} is a vector strings \code{"GENE_CODE:FROM_COMPOUND:TO_COMPOUND:REACTION_CODE:PATHWAY"} where:
\tabular{ll}{
\code{GENE_CODE}     \tab The name of the gene labeling the edge. \cr
\code{FROM_COMPOUND} \tab The start compound of the edge. \cr
\code{TO_COMPOUND}   \tab The end compound of the edge. \cr
\code{REACTION_CODE} \tab The reaction labeling the edge. \cr
\code{PATHWAY}       \tab The pathway where the edge is observed.
}

The returned \code{geneNetwork$edges} object is a \code{data.frame} with the following elements:
\tabular{ll}{
\code{from}  \tab The position of the from gene within the \code{nodes} object. \cr
\code{to}    \tab The position of the from gene within the \code{nodes} object. \cr
\code{label} \tab The label for the edge. \cr
\code{pathway} \tab The pathway where the edge was observed.
}

The \code{geneNetwork$edges$label} is the string \code{"FROM_COMPOUND:MIDDLE_COMPOUND:END_COMPOUND:PATHWAY"} where each element corresponds to:
\tabular{ll}{
\code{FROM_COMPOUND}   \tab KEGG compound code where the edge beings. \cr
\code{MIDDLE_COMPOUND} \tab KEGG middle compound that connect two genes along the edge. \cr
\code{TO_COMPOUND}     \tab KEGG compound code where the edge ends. \cr
}
}
\seealso{
\code{\link{getKEGG}}, \code{\link{pathRanker}}, \code{\link{assignEdgeWeights}}
}
\examples{
## See pathRanker help documentation for example useage
}

