\name{getKEGG}
\alias{getKEGG}
\title{Read or download KEGG KGML files and convert them to a compound network and reaction list.}
\description{Read or download KEGG KGML files and convert them to a compound network and reaction list.}

\usage{
kegg.obj <- getKEGG(maps = "all",organism = "",kegg.xml.location = "",url = "ftp://ftp.genome.jp/pub/kegg/xml/kgml/metabolic/organisms")
}
\arguments{
\item{maps}{A character list specifying the KEGG pathway map codes to be used.  To use all maps found within a local directory or on the ftp site set to maps = "all".}
\item{organism}{The name of the specific KEGG XML file to process.  Required for ftp download.}
\item{kegg.xml.location}{The local directory where the KEGG XML files are stored.  Set as "" for to download the maps from the kegg ftp site.}
\item{url}{The url of the ftp location for the kgml files.}
}
\value{
A list object with the following following items:
\tabular{ll}{
\code{edge.list} \tab An data frame of the edge list between compounds labelled by reaction. \cr
\code{reaction.info} \tab A list of each reaction, where each element contains the genes defining each reaction. \cr
\code{details} \tab The \code{data.frame} of the codes and names of all KEGG pathway maps merged.
}

The \code{edge.list} object is a \code{data.frame} where each row is:
\tabular{ll}{
\code{from} \tab Compound where the edge begins. \cr
\code{to} \tab Compound where the edge ends. \cr
\code{label} \tab Reaction which labels the edge. \cr
\code{pathway} \tab KEGG pathway map where the edge is observed.
}

The \code{reaction.info} object is a list named by each reaction and contains all genes that catalyze that reaction.

The \code{name} object is a list of all KEGG pathways contained within the network.

}
\details{
To read the XML files requires the \code{XML} package. To download the KEGG XML files requires \code{RCurl} package.

This function ignores all KEGG edges that are not compound->reaction->compound.  This includes simple compound->compound transitions and KEGG relation elements. 

Within a reaction all KEGG ortholog elements are removed.  If after all KEGG ortholog elements are removed the reaction is empty then the reaction is not considered.
}
\seealso{
\code{\link{computeGeneNetwork}}
}



